/*
 * Copyright 1990 MCNC and Vivek Khera
 *
 * Permission to use, copy, modify, distribute, and sell this software and its
 * documentation for any purpose is hereby granted without fee, provided that
 * the above copyright notice appear in all copies and that both that
 * copyright notice and this permission notice appear in supporting
 * documentation, and that the name of MCNC not be used in advertising or
 * publicity pertaining to distribution of the software without specific,
 * written prior permission.  MCNC makes no representations about the
 * suitability of this software for any purpose.  It is provided "as is"
 * without express or implied warranty.
 *
 * MCNC DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING ALL
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN NO EVENT SHALL MCNC
 * BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
 * WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION
 * OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN 
 * CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 *
 * Author:  Vivek Khera, MCNC
 */

/*
 * MCNC-KHERABEROS-1 authentication scheme. 
 *
 * authorization scheme based on Kerberos. requires the KACL extension. 
 *
 * V. Khera	2-AUG-1990
 * $Header: krbauth.c,v 1.3 90/09/18 10:17:00 bin Exp $ 
 */

#include "X.h"
#include "os.h"
#include <krb.h>

/*
 * define which kerberos service you wish to use. i use rcmd since it
 * is available everywhere.
 *
 * this must match the definition in lib/X/XOpenDis.c
 */
#ifndef SERVICE
#define SERVICE "rcmd"
#endif

static struct {
	unsigned short  len;
	char           *data;
	XID             id;
} krb_auth = {0, 0, -1};

/*
 * we only keep one cookie around, since we only need it for pre-kerberos
 * initialization from xdm. once names added to the access control list, the
 * cookie is not used anymore. 
 */
int
KrbAddCookie(data_length, data, id)
unsigned short data_length;
char *data;
XID id;
{
	char           *tmp;

	tmp = (char *) xrealloc(krb_auth.data, (unsigned) data_length);
	if (!tmp) {
		KrbResetCookie();
		return 0;
	} /* else */
	krb_auth.data = tmp;
	bcopy(data, krb_auth.data, (int) data_length);
	krb_auth.len = data_length;
	krb_auth.id = id;
	return 1;
}

/*
 * perform kerberos authentication on the data if there are names in the ACL,
 * otherwise, compare with the magic cookie stored. 
 */
XID
KrbCheckCookie(data_length, data)
unsigned short data_length;
char *data;
{
	extern int KACLInList();
	extern int KACLEmptyList();

	if (KACLEmptyList()) {
	/* 
	 * no names in list, so treat data as a magic cookie. this is 
	 * to allow xdm to pop up windows and to add the logged in 
	 * user to the acl. it is also a backup scheme if no kerberos
	 * tickets exist, and MCNC-KHERABEROS-1 is specified.
	 */
		if (data_length == krb_auth.len &&
		    bcmp(data, krb_auth.data, (int) data_length) == 0)
			return krb_auth.id;
		else
			return (XID) - 1;
	} else {
	/* 
	 * we must decode the kerberos authenticator and determine if 
	 * the user identified is in the ACL
	 */
		int i;
		KTEXT_ST ktxt;		/* Kerberos data */
		AUTH_DAT ad;		/* authentication data */
		char hostname[64];	/* for hostname */
		char authName[MAX_K_NAME_SZ+1];	/* authenticated name */

		gethostname(hostname, sizeof(hostname));
		bcopy(data, ktxt.dat, data_length);
		ktxt.length = data_length;
		i = krb_rd_req(&ktxt, SERVICE, hostname, 0, &ad, "");
		if (i != KSUCCESS)	/* failed authentication */
			return (XID) -1;
		/* now build up name from authenticator */
		sprintf(authName,"%s%s%s@%s", ad.pname, *ad.pinst ? "." : "",
			ad.pinst, ad.prealm);
		if (KACLInList(authName))
			return krb_auth.id;
		else
			return (XID) -1;
	}
}

int
KrbResetCookie()
{
	xfree(krb_auth.data);
	krb_auth.data = 0;
	krb_auth.len = 0;
	krb_auth.id = -1;
}

XID
KrbToID(data_length, data)
unsigned short data_length;
char *data;
{
	return krb_auth.id;
}

KrbFromID(id, data_lenp, datap)
XID id;
unsigned short *data_lenp;
char **datap;
{
	*data_lenp = krb_auth.len;
	*datap = krb_auth.data;
	return 1;
}

KrbRemoveCookie(data_length, data)
unsigned short data_length;
char *data;
{
	KrbResetCookie();	/* we only keep one around, anyway */
}
